local HideBlizzard = LibStub("AceAddon-3.0"):NewAddon("HideBlizzard", "AceConsole-3.0", "AceEvent-3.0")
HideBlizzard:SetEnabledState(true)
HideBlizzard:SetDefaultModuleState(false)
HideBlizzard:SetDefaultModuleLibraries("AceEvent-3.0")

local addonVersion = GetAddOnMetadata("HideBlizzard", "Version")
local db
local defaults = {
	profile = {
		modules = {
			ActionBar = false,
			Buttons = false,
			Pet = false,
			Player = false,
			SpecialFrames = false,
			Target = false,
			Vehicle = false,
		},
	},
}

function HideBlizzard:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("HideBlizzardDB", defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateView")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateView")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateView")
	db = self.db.profile

	local lastUpdate = 0
	local waitFrame = CreateFrame("Frame")
	waitFrame:SetScript("OnUpdate", function(self, elapsed)
		lastUpdate = lastUpdate + elapsed
		if(lastUpdate > 3) then
			if not db.Version then
				db.Version = addonVersion
				HideBlizzard:Print("Welcome! Type /hb to configure")
			elseif db.Version ~= addonVersion then
				if db.Version:match("1.0.59") then
					HideBlizzard:Print("|cffFF3333Notice: You need to ReloadUI() for HideBlizzard to work|r")
					HideBlizzardDB = {}
					waitFrame:SetScript("OnUpdate", nil)
				end
				db.Version = addonVersion
				HideBlizzard:Print(string.format("Updated to version |cff00ff66%s|r.", db.Version))
			end
			waitFrame:SetScript("OnUpdate", nil)
		end
	end)

	self:RegisterOptions()
end

function HideBlizzard:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_DISABLED", function()
		if InCombatLockdown() then
			if InterfaceOptionsFrame:IsVisible() then
				self:Print("Closing options window since you cannot set options while in combat")
				HideUIPanel(InterfaceOptionsFrame)
				HideUIPanel(GameMenuFrame)
			end
		end
	end)
end

function HideBlizzard:OnDisable()
end
-- @Mapster func
function HideBlizzard:GetModuleEnabled(module)
	return self.db.profile.modules[module]
end
-- @Mapster func
function HideBlizzard:SetModuleEnabled(module, value)
	local old = HideBlizzard.db.profile.modules[module]
	HideBlizzard.db.profile.modules[module] = value
	if old ~= value then
		if value then
			HideBlizzard:EnableModule(module)
		else
			HideBlizzard:DisableModule(module)
		end
	end
end
-- @Mapster func
function HideBlizzard:UpdateView()
	for k,v in self:IterateModules() do
		if self:GetModuleEnabled(k) and not v:IsEnabled() then
			self:EnableModule(k)
		elseif not self:GetModuleEnabled(k) and v:IsEnabled() then
			self:DisableModule(k)
		end
		if(type(v.UpdateView) == "function") then
			v:UpdateView()
		end
	end
end
